@@ -13,14 +13,15 @@ for i in range(100):
 X=Numeric.array(x_tmp)
 lines = matplotlib.matlab.plot(X[:,0],'o')
 
+manager = matplotlib.matlab.get_current_fig_manager()
 def updatefig(*args):
     updatefig.count += 1
     if updatefig.count>59: updatefig.count=0
     lines[0].set_data(ind,X[:,updatefig.count])
-    fig.draw()
+    manager.canvas.draw()
     return gtk.TRUE
 
 updatefig.count=-1
 
-gtk.timeout_add(200,updatefig)
+gtk.timeout_add(25,updatefig)
 matplotlib.matlab.show()
