@@ -1,9 +1,9 @@
 import matplotlib
 matplotlib.use('GTK')
 
-from matplotlib.backends import Figure
+from matplotlib.figure import Figure
 from matplotlib.axes import Subplot
-from matplotlib.backends.backend_gtk import NavigationToolbar
+from matplotlib.backends.backend_gtk import FigureCanvasGTK, NavigationToolbar
 
 from Numeric import arange, sin, pi
 import gtk
@@ -39,7 +39,6 @@ class WidgetsWrapper:
         self.widgets.signal_autoconnect(GladeHandlers.__dict__)
 
         self.figure = Figure(figsize=(8,6), dpi=72)
-        self.figure.show()
         self.axis = Subplot(self.figure, 111)
         self.figure.add_axis(self.axis)
         t = arange(0.0,3.0,0.01)
@@ -47,11 +46,14 @@ class WidgetsWrapper:
         self.axis.plot(t,s)
         self.axis.set_xlabel('time (s)')
         self.axis.set_ylabel('voltage')
-        
-        self['vboxMain'].pack_start(self.figure, gtk.TRUE, gtk.TRUE)
 
+        self.canvas = FigureCanvasGTK(self.figure) # a gtk.DrawingArea
+        self.canvas.show()
+        self['vboxMain'].pack_start(self.canvas, gtk.TRUE, gtk.TRUE)
+        self['vboxMain'].show()
+        
         # below is optional if you want the navigation toolbar
-        self.navToolbar = NavigationToolbar(self.figure, self['windowMain'])
+        self.navToolbar = NavigationToolbar(self.canvas, self['windowMain'])
         self.navToolbar.lastDir = '/var/tmp/'
         self['vboxMain'].pack_start(self.navToolbar)
         self.navToolbar.show()
