@@ -1,11 +1,14 @@
 # note, this is currentl *very slow*.  I'm working on improving the
 # efficiency of pcolor.  If you want to run this example and your
 # computer is not very fast, I suggest you grab a cup of coffee while
-# it runs
+# it runs.  Look for major improvments in pcolor in future releases of
+# matplotlib when we add image handling capability in extension module
+# code
 
+from __future__ import division
 from matplotlib.matlab import *
 from matplotlib.lines import Line2D
-
+from matplotlib.transforms import Transform, Bound1D
 # I use if 1 to break up the different regions of code visually
 
 if 1:   # load the data
@@ -40,17 +43,28 @@ if 1:   # plot the EEG
     t = arange(numSamples)/float(numSamples)*10.0
     ticklocs = []
     ax = subplot(212)
-    for i in range(numRows):
-        thisLine = Line2D(ax.dpi, ax.bbox, t, data[:,i],
-                          transx=ax.xaxis.transData, transy=ax.yaxis.transData)
-        thisLine.set_vertical_offset(3*i)
-        ax.add_line(thisLine)
-        ticklocs.append(3*i)
 
+    height = 72  # height of one EEG in pixels
+    # transform data to axes coord (0,1)
+    transy = Transform(Bound1D(-.05,.05), Bound1D(-.2,.2))
+    for i in range(numRows):
+        thisLine = Line2D(
+            ax.dpi, ax.bbox, t, data[:,i]-data[0,i],
+            transx=ax.xaxis.transData,
+            transy=transy)
+        offset = (i+1)/(numRows+1)
+        thisLine.set_vertical_offset(offset, ax.yaxis.transAxis)
+
+        ax.add_line(thisLine)
+        ticklocs.append(offset)
+
+    set(gca(), 'xlim', [0,10])
     set(gca(), 'xticks', arange(10))
-    set(gca(), 'yticks', ticklocs)
-    set(gca(), 'yticklabels', ['PG3', 'PG5', 'PG7', 'PG9'])
-    set(gca(), 'ylim', [-3, 12])
+    yticks = set(gca(), 'yticks', ticklocs)
+    set(gca(), 'yticklabels', ['PG3', 'PG5', 'PG7', 'PG9']) 
+
+    # set the yticks to use axes coords on the y axis
+    set(yticks, 'transform', ax.yaxis.transAxis)
     xlabel('time (s)')
 
 
