@@ -10,15 +10,18 @@ t2 = arange(0.0, 5.0, 0.02)
 t3 = arange(0.0, 2.0, 0.01)
 
 subplot(211)
-plot(t1, f(t1), 'bo', t2, f(t2), 'k')
+l = plot(t1, f(t1), 'bo', t2, f(t2), 'k')
+set(l, 'markerfacecolor', 'g')
+grid(True)
 title('A tale of 2 subplots')
 ylabel('Damped oscillation')
 
 subplot(212)
-plot(t3, cos(2*pi*t3), 'r--')
+plot(t3, cos(2*pi*t3), 'r>')
+grid(True)
 xlabel('time (s)')
 ylabel('Undamped')
 
-#savefig('subplot_demo.eps')
+#savefig('subplot_demo', dpi=300)
 show()
 
