@@ -15,9 +15,9 @@ t2 = arange(0.0, 5.0, 0.02)
 
 plot(t1, f(t1), 'bo', t2, f(t2), 'k')
 title('Damped exponential decay', font, fontsize=14, color='r')
-text(3, 0.65, 'cos(2 pi t) exp(-t)', font, color='k')
-xlabel('time (s)', font)
+text(2, 0.65, 'cos(2 pi t) exp(-t)', font, color='k')
+xlabel('time (s)', font, fontangle='italic')
 ylabel('voltage (mV)', font)
 
-savefig('text_themes', dpi=150)
+#savefig('text_themes')
 show()
