@@ -48,10 +48,10 @@ class Tick(Artist):
         Artist.__init__(self, axes.dpi, axes.bbox)
         self.axes = axes
 
-        self.tick1line = self._get_default_tick1line(loc, size)
-        self.tick2line = self._get_default_tick2line(loc, size)
-        self.gridline = self._get_default_gridline(loc)        
-        self.label = self._get_default_text(loc, label)
+        self.tick1line = self._get_tick1line(loc, size)
+        self.tick2line = self._get_tick2line(loc, size)
+        self.gridline = self._get_gridline(loc)        
+        self.label = self._get_text(loc, label)
         self.label.set_clip_on( False )
         self.gridOn = gridOn
         self.tick1On = tick1On
@@ -60,19 +60,19 @@ class Tick(Artist):
         self._loc = loc
         self._size = size
         
-    def _get_default_text(self, loc, label):
+    def _get_text(self, loc, label):
         'Get the default Text instance'
         raise NotImplementedError('Derived must override')
 
-    def _get_default_tick1line(self, loc, size):
+    def _get_tick1line(self, loc, size):
         'Get the default line2D instance for tick1'
         raise NotImplementedError('Derived must override')
 
-    def _get_default_tick2line(self, loc, size):
+    def _get_tick2line(self, loc, size):
         'Get the default line2D instance for tick2'
         raise NotImplementedError('Derived must override')
 
-    def _get_default_gridline(self, loc):
+    def _get_gridline(self, loc):
         'Get the default grid Line2d instance for this tick'
         raise NotImplementedError('Derived must override')
 
@@ -90,6 +90,7 @@ class Tick(Artist):
         if midPoint and self.gridOn:  self.gridline.draw(renderer)
         if self.labelOn: self.label.draw(renderer)
 
+    
     def set_loc(self, loc):
         'Set the location of tick in data coords with scalar loc'
         raise NotImplementedError('Derived must override')
@@ -102,13 +103,35 @@ class Tick(Artist):
         'return true if tickloc is not on axes boundary'
         raise NotImplementedError('Derived must override')
 
+    def set_transform(self, trans):
+        """
+        Set the transform for the dimension the ticks vary along.  Eg,
+        for xticks, this is the x dimension and for y ticks this is
+        the y dimension.  All the child artists transforms will be ste
+        """
+        raise NotImplementedError('Derived must override')
+
 class XTick(Tick):
     """
     Contains all the Artists needed to make an x tick - the tick line,
     the label text and the grid line
     """
 
-    def _get_default_text(self, loc, label):
+    def set_transform(self, trans):
+        """
+        Set the transform for the x dimension the ticks vary along.  e
+        """
+
+        self.tick1line.transx = trans
+        self.tick2line.transx = trans
+        self.gridline.transx = trans
+        self.label.transx = trans
+
+
+
+            
+    
+    def _get_text(self, loc, label):
         'Get the default Text instance'
         # the y loc is 3 points below the min of y axis
         top = self.bbox.y.get_refmin() - self.dpi*RRef(3/72.0)
@@ -116,46 +139,49 @@ class XTick(Tick):
             dpi=self.dpi, 
             bbox=self.bbox,
             x=loc,
-            y=top,  
+            y=top,
             fontsize=10,
             verticalalignment='top',
             horizontalalignment='center',
             transx = self.axes.xaxis.transData,
-            # transy is default, idenity transform
+            # transy is default, identity transform
             )
         return text
 
         
-    def _get_default_tick1line(self, loc, size):
+    def _get_tick1line(self, loc, size):
         'Get the default line2D instance'
 
         line = Line2D( self.dpi, self.bbox,
                        xdata=(loc, loc), ydata=(0, size),
                        color='k',
+                       antialiased=False, 
                        transx = self.axes.xaxis.transData,
                        transy = self.axes.yaxis.get_pts_transform(),
                        )
         line.set_data_clipping(False)
         return line
 
-    def _get_default_tick2line(self, loc, size):
+    def _get_tick2line(self, loc, size):
         'Get the default line2D instance'
         offset = self.axes.yaxis.displaylim.get_refmax()
         ytrans = self.axes.yaxis.get_pts_transform(offset)
         line = Line2D( self.dpi, self.bbox,
                        xdata=(loc, loc), ydata=(-size, 0),
                        color='k',
+                       antialiased=False, 
                        transx = self.axes.xaxis.transData,
                        transy = ytrans,
                        )
         line.set_data_clipping(False)
         return line
 
-    def _get_default_gridline(self, loc):
+    def _get_gridline(self, loc):
         'Get the default line2D instance'
         line = Line2D( self.dpi, self.bbox,
                        xdata=(loc, loc), ydata=(0, 1),
                        color='k', linestyle=':',
+                       antialiased=False, 
                        transx = self.axes.xaxis.transData,
                        transy = self.axes.yaxis.transAxis,
                        )
@@ -180,15 +206,25 @@ class YTick(Tick):
     Contains all the Artists needed to make a Y tick - the tick line,
     the label text and the grid line
     """
+    def set_transform(self, trans):
+        """
+        Set the transform for the y dimension the ticks vary along.  e
+        """
+
+        self.tick1line.transy = trans
+        self.tick2line.transy = trans
+        self.gridline.transy = trans
+        self.label.transy = trans
+
     # how far from the y axis line the right of the ticklabel are
-    def _get_default_text(self, loc, label):
+    def _get_text(self, loc, label):
         'Get the default Text instance'
         right = self.bbox.x.get_refmin() - self.dpi*RRef(3/72.0)
         return Text(
             dpi=self.dpi, 
             bbox=self.bbox,
             x=right,
-            y=loc,
+            y=loc,            
             fontsize=10,
             verticalalignment='center',
             horizontalalignment='right',
@@ -198,33 +234,39 @@ class YTick(Tick):
             )
         return text
 
-    def _get_default_tick1line(self, loc, size):
+    def _get_tick1line(self, loc, size):
         'Get the default line2D instance'
         line = Line2D( self.dpi, self.bbox,
                        (0, size), (loc, loc), color='k',
+                       antialiased=False, 
                        transx = self.axes.xaxis.get_pts_transform(),
                        transy = self.axes.yaxis.transData,
                        )
         line.set_data_clipping(False)
         return line
 
-    def _get_default_tick2line(self, loc, size):
+    def _get_tick2line(self, loc, size):
         'Get the default line2D instance'
-        offset = self.axes.xaxis.displaylim.get_refmax()
+
+        # size points to the left of the xaxis max
+        offset = self.axes.xaxis.displaylim.get_refmax() - self.dpi*RRef(size/72.0)
         xtrans = self.axes.xaxis.get_pts_transform(offset)
 
         line = Line2D( self.dpi, self.bbox,
-                       (1-size,1), (loc, loc), color='k',
+                       (0, size), (loc, loc), color='k',
+                       antialiased=False, 
                        transx = xtrans,
                        transy = self.axes.yaxis.transData,
                        )
+        line.set_data_clipping(False)
         return line
 
-    def _get_default_gridline(self, loc):
+    def _get_gridline(self, loc):
         'Get the default line2D instance'
         line = Line2D( self.dpi, self.bbox,
                        xdata=(0,1), ydata=(loc,loc), 
                        color='k', linestyle=':',
+                       antialiased=False, 
                        transx = self.axes.xaxis.transAxis,
                        transy = self.axes.yaxis.transData,
                        )
@@ -244,6 +286,16 @@ class YTick(Tick):
         'return true if tickloc is not on axes boundary'
         return self.axes.yaxis.viewlim.in_open_interval(loc)
 
+def decade_down(x):
+    'floor x to the nearest lower decade'
+    lx = math.floor(math.log10(x))
+    return 10**lx
+
+def decade_up(x):
+    'ceil x to the nearest higher decade'
+    lx = math.ceil(math.log10(x))
+    return 10**lx
+
     
 class Axis(Artist):
 
@@ -302,15 +354,21 @@ class Axis(Artist):
             vmax = math.ceil(scale*vmax)/scale
 
             self.viewlim.set_bounds(vmin, vmax)
+        elif self._scale=='log':
+            if vmin==vmax:
+                vmin = decade_down(vmin)
+                vmax = decade_up(vmax)
+                self.viewlim.set_bounds(vmin, vmax)
+                
 
     def build_artists(self):
         """
         Call only after xaxis and yaxis have been initialized in axes
         since the trasnforms require both
         """
-        self._label = self._get_default_label()
+        self._label = self._get_label()
         self._label.set_clip_on( False )
-        self._ticks = [self._get_default_tick() for i in range(20)]
+        self._ticks = [self._get_tick() for i in range(20)]
 
     def _draw(self, renderer, *args, **kwargs):
         'Draw the axis lines, grid lines, tick lines and labels'
@@ -327,13 +385,17 @@ class Axis(Artist):
         else:
             labels = [self.format_tickval(loc) for loc in locs]
 
+
+
         ticklabelBoxes = []
         for tick, loc, label in zip(ticks, locs, labels):
             if not self.viewlim.in_interval(loc): continue
             tick.set_loc(loc)
             tick.set_label(label)
-            tick.draw(renderer)
-            ticklabelBoxes.append(tick.label.get_window_extent(renderer))
+
+            tick.draw(renderer)  # cpu leak, but no mem leak here
+            extent = tick.label.get_window_extent(renderer) # fixed cpu leak by cacheing extent
+            ticklabelBoxes.append(extent)
 
 
         # find the tick labels that are close to the axis labels.  I
@@ -346,8 +408,10 @@ class Axis(Artist):
         labelBox.y.scale(1.25)
         overlaps = [b for b in ticklabelBoxes if b.overlap(labelBox)]
 
+
         self._update_label_postion(overlaps)
-        self._label.draw(renderer)
+
+        self._label.draw(renderer)  # memory leak here, vertical text
 
         if 0: # draw the bounding boxes around the text for debug
             for tick in ticks:
@@ -369,15 +433,19 @@ class Axis(Artist):
         if abs(x)<1e4 and x==int(x): return '%d' % x
 
         # if the value is just a fraction off an int, use the int
-        if abs(x-int(x))<0.0001*d: return '%d' % int(x)
+        if abs(x-int(x))<0.0001*d: return '%d' % int(x + 0.5)
 
         # use exponential formatting for really big or small numbers,
         # else use float
-        if abs(x) < 1e-4: fmt = '%1.3e'
-        elif abs(x) > 1e5: fmt = '%1.3e'
+
+        if d < 1e-2: fmt = '%1.2e'
+        elif d < 1e-1: fmt = '%1.2f'
+        elif d > 1e5: fmt = '%1.3e'
+        elif d > 10 : fmt = '%1.1f'
+        elif d > 1 : fmt = '%1.2f'
         else: fmt = '%1.3f'
         s =  fmt % x
-
+        #print d, fmt, x, s
         # strip trailing zeros, remove '+', and handle exponential formatting
         m = self._zerorgx.match(s)
         if m:
@@ -409,7 +477,7 @@ class Axis(Artist):
         dots =  Dots( self.dpi)
         return TransformSize(cm, dots, offset, transOffset)
 
-    def _get_default_label(self):
+    def _get_label(self):
         raise NotImplementedError('Derived must override')
 
     def _get_display_lim(self):
@@ -543,7 +611,7 @@ class Axis(Artist):
         """
         if len(self._ticks)<numticks:
             for i in range(numticks-len(self._ticks)):
-                tick = self._get_default_tick()
+                tick = self._get_tick()
                 if self._gridOn: tick.gridOn = True
                 for attr, val in self._setChildAttrs.items():
                     tick.set_child_attr(attr, val)
@@ -597,7 +665,8 @@ class Axis(Artist):
         'Set the locations of the tick marks from sequence ticks'
         self._ticklocs = asarray(ticks)
         self.viewlim.update(self._ticklocs)
-
+        return self.get_ticks()
+    
     def _update_label_postion(self, overlaps):
         """
         Update the label position based on the sequence of bounding
@@ -617,14 +686,14 @@ class Axis(Artist):
 class XAxis(Axis):
     __name__ = 'xaxis'
 
-    def _get_default_tick(self):
+    def _get_tick(self):
         return XTick(self.axes, 0, '')
 
-    def _get_default_label(self):
+    def _get_label(self):
         label = Text(
             self.dpi, 
             self.bbox, 
-            x = 0.5,       # centered
+            x = 0.5,               # centered
             y = self.LABELPAD,     # rel coords under axis line
             fontsize=12,
             verticalalignment='top',
@@ -655,11 +724,11 @@ class XAxis(Axis):
 class YAxis(Axis):
     __name__ = 'yaxis'
 
-    def _get_default_tick(self):
+    def _get_tick(self):
         return YTick(self.axes, 0, '')
 
 
-    def _get_default_label(self):
+    def _get_label(self):
         label = Text(
             dpi=self.dpi,
             bbox = self.bbox, 
