@@ -16,25 +16,24 @@ from gtk import gdk
 import pango
 
 import matplotlib
-from Numeric import array, Int16
+from Numeric import Int16, asarray
 
-from matplotlib.cbook import iterable, is_string_like, flatten, \
-     enumerate, True, False
-from matplotlib.transforms import Transform, Bound1D, Bound2D
-from matplotlib.backend_bases import RendererBase, GraphicsContextBase,\
-     FigureBase, FigureManagerBase
+from matplotlib.cbook import is_string_like,  enumerate, True, False
+from matplotlib.transforms import Bound2D
+
+from matplotlib.backend_bases import \
+     RendererBase, GraphicsContextBase, FigureManagerBase, FigureCanvasBase
 from matplotlib._matlab_helpers import Gcf
-from matplotlib.artist import Artist, DPI
-from matplotlib.text import Text
+from matplotlib.figure import Figure
 
-# enable ps save from gtk backend
-from matplotlib import switch_backends
-from backend_ps import FigurePS
-from copy import deepcopy
 
 # the true dots per inch on the screen; should be display dependent
 # see http://groups.google.com/groups?q=screen+dpi+x11&hl=en&lr=&ie=UTF-8&oe=UTF-8&safe=off&selm=7077.26e81ad5%40swift.cs.tcd.ie&rnum=5 for some info about screen dpi
-SCREENDPI = 75
+PIXELS_PER_INCH = 72
+
+def round(x):
+    return int(math.floor(x+0.5))
+
 
 class ColorManagerGTK:
     _cached = {}  # a map from get_color args to colors
@@ -85,20 +84,27 @@ class RendererGTK(RendererBase):
         'oblique' : pango.STYLE_OBLIQUE,
         }
 
-
+    # cache for efficiency, these must be at class, not instance level
+    layoutd = {}  # a map from text prop tups to pango layouts
+    extentd = {}  # a map from text prop tups to text extents
+    offsetd = {}  # a map from text prop tups to text offsets
+    rotated = {}  # a map from text prop tups to rotated text pixbufs
     def __init__(self, gtkDA, gdkDrawable, dpi):
         self.gtkDA = gtkDA
         self.gdkDrawable = gdkDrawable
         self.width, self.height = self.gdkDrawable.get_size()
         self.dpi = dpi
-        self.layoutd = {}  # a map from text prop tups to pango layouts
 
-    def compute_text_offsets(self, t):
+    def compute_text_offsets(self, t, props=None):
         """
         Return the (x,y) offsets to adjust for the alignment
         specifications
         """
 
+        if props is None:
+            props = t.get_prop_tup()
+        tup = self.offsetd.get(props)
+        if tup is not None: return tup
         layout = self.get_pango_layout(t)
 
         inkRect, logicalRect = layout.get_pixel_extents()
@@ -130,22 +136,25 @@ class RendererGTK(RendererBase):
             elif valign=='top': offsety = 0
             else: offsety = h
 
-        return (offsetx-l, offsety+b)
+        tup = offsetx-l, offsety+b
+        self.offsetd[props] = tup
+        return tup
 
     
-    def draw_arc(self, gc, faceColor, x, y, width, height, angle1, angle2):
+    def draw_arc(self, gc, rgbFace, x, y, width, height, angle1, angle2):
         """
         Draw an arc centered at x,y with width and height
         """
         x, y = int(x-0.5*width), self.height-int(y+0.5*height)
         w, h = int(width)+1, int(height)+1
         a1, a2 = int(angle1*64), int(angle2*64)
-
-        if faceColor is not None:            
-            rgb = gc.get_rgb()
-            gc.set_foreground(faceColor)
+        
+        if rgbFace is not None:
+            edgecolor = gc.gdkGC.foreground
+            facecolor = colorManager.get_color(rgbFace)
+            gc.gdkGC.foreground = facecolor
             self.gdkDrawable.draw_arc(gc.gdkGC, True, x, y, w, h, a1, a2)
-            gc.set_foreground(rgb)
+            gc.gdkGC.foreground = edgecolor
         self.gdkDrawable.draw_arc(gc.gdkGC, False, x, y, w, h, a1, a2)
             
     def draw_line(self, gc, x1, y1, x2, y2):
@@ -169,63 +178,50 @@ class RendererGTK(RendererBase):
         self.gdkDrawable.draw_point(
             gc.gdkGC, int(x), self.height-int(y))
 
-    def draw_polygon(self, gc, faceColor, points):
+    def draw_polygon(self, gc, rgbFace, points):
         """
         Draw a polygon.  points is a len vertices tuple, each element
         giving the x,y coords a vertex
 
-        If faceColor is not None, fill the rectangle with it.  gcEdge
+        If gcFace is not None, fill the rectangle with it.  gcEdge
         is a GraphicsContext instance
 
         """
         points = [(int(x), self.height-int(y)) for x,y in points]
-        if faceColor is not None:
-            rgb = gc.get_rgb()  #cache color
-            gc.set_foreground(faceColor)
-            self.gdkDrawable.draw_polygon(gc.gdkGC, False, points)
-            gc.set_foreground(rgb)  # restore color
+        if rgbFace is not None:
+            edgecolor = gc.gdkGC.foreground
+            facecolor = colorManager.get_color(rgbFace)
+            gc.gdkGC.foreground = facecolor
+            self.gdkDrawable.draw_polygon(gc.gdkGC, True, points)
+            gc.gdkGC.foreground = edgecolor
+
         self.gdkDrawable.draw_polygon(gc.gdkGC, True, points)
 
 
-    def draw_rectangle(self, gc, faceColor, x, y, width, height):
+    def draw_rectangle(self, gc, rgbFace, x, y, width, height):
         """
         Draw a rectangle at lower left x,y with width and height
         If filled=True, fill the rectangle with the gc foreground
         gc is a GraphicsContext instance
         """
-        
+
         #x, y = int(x), self.height-int(y+height)
         #w, h = int(width), int(height)
         x, y = int(x), self.height-int(math.ceil(y+height))
         w, h = int(math.ceil(width)), int(math.ceil(height))
 
 
-        needEdge=True
-
-        # this is a hack.  high res pcolor was mindnumbingly slow and
-        # I found some bottlenecks here in the profiler. I am trying
-        # to do some speedups by avoiding the proper function calls
-        # and using protected attributes in their place.  Work on
-        # improving this!
-        if faceColor is not None:
-            # cache color; using protected attribute for performace.
-            # me = bad, bad
-            oldRGB = gc._rgb  
-            oldFG = gc.gdkGC.foreground
-
-            
-            gc.set_foreground(faceColor)
+        if rgbFace is not None:
+            edgecolor = gc.gdkGC.foreground
+            facecolor = colorManager.get_color(rgbFace)
+            gc.gdkGC.foreground = facecolor
             self.gdkDrawable.draw_rectangle(gc.gdkGC, True, x, y, w, h)
-            if gc._rgb==oldRGB: needEdge = False 
+            gc.gdkGC.foreground = edgecolor
+            
+        self.gdkDrawable.draw_rectangle(gc.gdkGC, False, x, y, w, h)
 
-            # restore old colors
-            gc._rgb = oldRGB
-            gc.gdkGC.foreground = oldFG
 
-        if needEdge:
-            self.gdkDrawable.draw_rectangle(gc.gdkGC, False, x, y, w, h)
-
-    def _draw_rotated_text(self, x, y, t, layout, gc):
+    def _draw_rotated_text(self, x, y, t, layout, gc, props):
         """
         Draw the text rotated 90 degrees
         """
@@ -239,69 +235,63 @@ class RendererGTK(RendererBase):
         w, h = int(rect[2]), int(rect[3])
 
         # get the background image
-        ox, oy = self.compute_text_offsets(t)
+        ox, oy = self.compute_text_offsets(t, props)
         y = self.height-y
 
         xox = max(0,int(x+ox))
         yoy = max(0,int(y+oy))
 
-        imgBack = gdrawable.get_image(xox, yoy, h, w)
-        if imgBack is None: return 0
+        # todo: cache rotation for dynamic redraw until pygtk mem leak
+        # fixed
+        imageOut = self.rotated.get(props)
+        if imageOut is not None:
+            gdrawable.draw_image(ggc, imageOut, 0, 0, xox, yoy, h, w)
+            return
 
-        pixmap = gtk.gdk.Pixmap(self.gtkDA.window, w, h)
-        
+        # save the background
+        imageBack = gdrawable.get_image(xox, yoy, w, h)
+        imageVert = gdrawable.get_image(xox, yoy, h, w)
 
-        # rotate the background image to horizontal to fill the pixmap
-        # background
-        imageHoriz = gtk.gdk.Image(type=0,
-                                   visual=pixmap.get_visual(),
-                                   width=w, height=h)
-        imageHoriz.set_colormap(imgBack.get_colormap())
+        # transform the vertical image, write it onto the renderer,
+        # and draw the layout onto it
+        imageFlip = gtk.gdk.Image(type=gdk.IMAGE_NORMAL,
+                                  visual=gdrawable.get_visual(),
+                                  width=w, height=h)
+        imageFlip.set_colormap(gdrawable.get_colormap())
         for i in range(w):
             for j in range(h):
-                imageHoriz.put_pixel(i, j, imgBack.get_pixel(j,w-i-1) )
-                
+                imageFlip.put_pixel(i, j, imageVert.get_pixel(j,w-i-1) )
 
+        gdrawable.draw_image(ggc, imageFlip, 0, 0, xox, yoy, w, h)
+        gdrawable.draw_layout(ggc, xox, yoy, layout)
 
-
-        pixmap.draw_image(ggc, imageHoriz, 0, 0, 0, 0, w, h)
-
-        pixmap.draw_layout(ggc, x=0, y=0, layout=layout)
-        imageIn = pixmap.get_image(x=0, y=0, width=w, height=h)
-        imageOut = gtk.gdk.Image(type=0,
-                                 visual=pixmap.get_visual(),
+        # now get that image and flip it vertical
+        imageIn = gdrawable.get_image(xox, yoy, w, h)
+        imageOut = gtk.gdk.Image(type=gdk.IMAGE_NORMAL,
+                                 visual=gdrawable.get_visual(),
                                  width=h, height=w)
-        imageOut.set_colormap(imageIn.get_colormap())
+        imageOut.set_colormap(gdrawable.get_colormap())
         for i in range(w):
             for j in range(h):
                 imageOut.put_pixel(j, i, imageIn.get_pixel(w-i-1,j) )
 
-
-        pixbuf = gtk.gdk.Pixbuf(colorspace=gtk.gdk.COLORSPACE_RGB,
-                                has_alpha=0, bits_per_sample=8,
-                                width=h, height=w)
-        pixbuf.get_from_image(src=imageOut, cmap=imageIn.get_colormap(),
-                              src_x=0, src_y=0, dest_x=0, dest_y=0,
-                              width=h, height=w)
-
-        pixbuf.render_to_drawable(gdrawable, ggc,
-                                  src_x=0, src_y=0,
-                                  dest_x=xox, dest_y=yoy,
-                                  width=h, height=w,
-                                  dither=0, x_dither=0, y_dither=0)
+        # draw the old background and the flipped text
+        gdrawable.draw_image(ggc, imageBack, 0, 0, xox, yoy, w, h)
+        gdrawable.draw_image(ggc, imageOut, 0, 0, xox, yoy, h, w)
+        self.rotated[props] = imageOut
         return True
 
 
-
     def draw_text(self, gc, x, y, t):
         if t.get_text()=='': return
 
         layout = self.get_pango_layout(t)
 
-        ox, oy = self.compute_text_offsets(t)
+        props = t.get_prop_tup()
+        ox, oy = self.compute_text_offsets(t, props)
 
         if t.get_rotation()=='vertical':            
-            drawn = self._draw_rotated_text(x, y, t, layout, gc)
+            drawn = self._draw_rotated_text(x, y, t, layout, gc, props)
         else:
             xox = int(x+ox)
             yoy = self.height - int(y+oy)
@@ -321,11 +311,15 @@ class RendererGTK(RendererBase):
         layout = self.layoutd.get(props)
         if layout is not None: return layout
 
+        fontname = t.get_fontname()
+        if fontname.lower()=='times': fontname = "serif"
+            
         font = pango.FontDescription(
-            '%s' % t.get_fontname())
+            '%s' % fontname)
         font.set_weight(self.fontweights[t.get_fontweight()])
         font.set_style(self.fontangles[t.get_fontangle()])
-        scale = t.dpi.get()/72.0
+
+        scale = self.get_text_scale(t)
         font.set_size(int(scale*t.get_fontsize()*1024))
         context = self.gtkDA.create_pango_context()
         layout  = self.gtkDA.create_pango_layout(t.get_text())
@@ -338,13 +332,18 @@ class RendererGTK(RendererBase):
     def get_text_extent(self, t):
         'Return the text extent in display coords as a Bound2D instance'
 
+        props = t.get_prop_tup()
+        extent = self.extentd.get(props)
+
+        if extent is not None: return extent
+
         layout = self.get_pango_layout(t)
         
         inkRect, logicalRect = layout.get_pixel_extents()
         rect = inkRect
         l,b,w,h = rect
         x, y  = t.get_xy_display()
-        ox, oy = self.compute_text_offsets(t)
+        ox, oy = self.compute_text_offsets(t, props)
         x = x+ox
         y = y+oy
 
@@ -357,12 +356,27 @@ class RendererGTK(RendererBase):
             left = x+l
             bottom = y-h-b
 
-        return Bound2D(left, bottom, w, h)
-        
+        extent =  Bound2D(left, bottom, w, h)
+        self.extentd[props] = extent
+        return extent
+
+    def get_text_scale(self, t):
+        """
+        Return the scale factor for fontsize taking screendpi and pixels per
+        inch into account
+        """
+        return t.dpi.get()/PIXELS_PER_INCH
 
     def new_gc(self):
         return GraphicsContextGTK(self.gdkDrawable.new_gc(), self)
 
+    def points_to_pixels(self, points):
+        """
+        convert point measures to pixes using dpi and the pixels per
+        inch of the display
+        """
+        return points*PIXELS_PER_INCH/72.0*self.dpi.get()/72.0
+
 
 class GraphicsContextGTK(GraphicsContextBase):
 
@@ -379,26 +393,32 @@ class GraphicsContextGTK(GraphicsContextBase):
         }
 
               
-    def __init__(self, gdkGC, drawable):
+    def __init__(self, gdkGC, renderer):
             GraphicsContextBase.__init__(self)
             self.gdkGC = gdkGC
-            self.drawable = drawable
+            self.renderer = renderer
+
 
     def set_clip_rectangle(self, rectangle):
         GraphicsContextBase.set_clip_rectangle(self, rectangle)
         l,b,w,h = rectangle
-        rectangle = (int(l), self.drawable.height-int(b+h),
+        rectangle = (int(l), self.renderer.height-int(b+h),
                      int(w), int(h))
         self.gdkGC.set_clip_rectangle(rectangle)        
 
 
     def set_dashes(self, dash_offset, dash_list):
         GraphicsContextBase.set_dashes(self, dash_offset, dash_list)
-        dpi = self.drawable.dpi.get()
+
+        def one_or_round(val):
+            if val<1: return 1
+            else: return round(val)
+
         if dash_list is not None:
-            dashes = dash_list*(dpi/72.0)
+            pixels = self.renderer.points_to_pixels(asarray(dash_list))
             self.gdkGC.line_style = gdk.LINE_ON_OFF_DASH
-            dl = [int(math.ceil(val)) for val in dash_list]
+            
+            dl = [one_or_round(val) for val in pixels]
             self.gdkGC.set_dashes(dash_offset, dl)
         else:
             self.gdkGC.line_style = gdk.LINE_SOLID
@@ -420,15 +440,17 @@ class GraphicsContextGTK(GraphicsContextBase):
         self.gdkGC.foreground = self._get_gdk_color()
         
 
-    def set_linewidth(self, w):
-        if w>0 and w<1: w = 1
-        GraphicsContextBase.set_linewidth(self, w)
-        self.gdkGC.line_width = int(self._linewidth)
+    def set_linewidth(self, lw):
+        GraphicsContextBase.set_linewidth(self, lw)
+
+        pixels = self.renderer.points_to_pixels(lw)
+        if pixels<1: pixels = 1
+        else: pixels = round(pixels)
+
+        self.gdkGC.line_width = pixels
 
     def set_linestyle(self, style):
         GraphicsContextBase.set_linestyle(self, style)
-        offset, dashes = self._dashd[style]
-        self.set_dashes(offset, dashes)
                                                
     def set_capstyle(self, cs):
         """
@@ -452,239 +474,9 @@ class GraphicsContextGTK(GraphicsContextBase):
         lw = self._linewidth
         js = self._joind[self._joinstyle]
         cs = self._capd[self._capstyle]
-        
-class FigureGTK(FigureBase, gtk.DrawingArea):
-    def __init__(self, figsize, dpi):
-        dpi = 72.0/SCREENDPI*dpi
-        FigureBase.__init__(self, figsize, dpi)
-        gtk.DrawingArea.__init__(self)
-        self.set_double_buffered(False)
-        w = self.bbox.x.interval()
-        h = self.bbox.y.interval()
-        self.set_size_request(int(w), int(h))
-
-        #self.connect('focus_in_event', self.focus_in_event)
-        self.connect('key_press_event', self.key_press_event)
-        self.connect('key_release_event', self.key_release_event)
-        self.connect('expose_event', self.expose_event)
-        self.connect('configure_event', self.configure_event)
-        self.connect('realize', self.realize)
-        self.connect('motion_notify_event', self.motion_notify_event)
-        self.connect('button_press_event', self.button_press_event)
-        self.connect('button_release_event', self.button_release_event)
-
-        self.set_events(
-            #gdk.FOCUS_CHANGE_MASK|
-            gdk.KEY_PRESS_MASK|
-            gdk.KEY_RELEASE_MASK|
-            gdk.EXPOSURE_MASK |
-            gdk.LEAVE_NOTIFY_MASK |
-            gdk.BUTTON_PRESS_MASK |
-            gdk.BUTTON_RELEASE_MASK |
-            gdk.POINTER_MOTION_MASK )
-        
-
-        self._inExpose = False
-        self._isConfigured = False
-        self._isRealized = False
-        self._printQued = []
-
-        self.grey = ( 0.7, 0.7, 0.7)
-        self._drawable = None
-        self._doplot = True    # if False don't plot, just print
-        self._pixmap = None
-
-    def set_do_plot(self, b):
-        self._doplot = b
-        
-    def button_press_event(self, widget, event):
-        pass
-    def button_release_event(self, widget, event):
-        pass
-    def motion_notify_event(self, widget, event):
-        pass
-    def key_press_event(self, widget, event):
-        pass
-    def key_release_event(self, widget, event):
-        pass
-        
-        
-    def configure_event(self, widget, event=None):
-
-        w,h = widget.window.get_size()
-        if w==1 or h==1: return # empty fig
-        # handle window resizes properly
-        if not self._isConfigured:
-            self._origWidth, self._origHeight = w, h
-
-        self.bbox.set_bounds(0, 0, w, h)
-        scalew = w/self._origWidth
-        scaleh = h/self._origHeight
-
-        for a in self.axes:
-            a.resize()
-
-        if self._isConfigured and self._isRealized and self._doplot:
-            self.draw()
-
-        self._isConfigured = True
-        return gtk.TRUE
-        
-
-    def draw(self, drawable=None, *args, **kwargs):
-
-        if drawable is None: drawable=self.renderer
-        if drawable is None: return 
-        self.renderer = drawable
-        if not self._isRealized: return 
-
-        gc = drawable.new_gc()
-
-        width, height = int(self.bbox.x.interval()), int(self.bbox.y.interval())
-
-        pixmap = RendererGTK(self, gtk.gdk.Pixmap(
-            self.renderer.gdkDrawable,
-            width, height), self.dpi)
-                             
-        if self._inExpose:
-            # override the clip
-            gc.set_clip_rectangle(self.bbox.get_bounds())
-            
-        gc.set_foreground(self.grey)
-        pixmap.gdkDrawable.draw_rectangle(
-            gc.gdkGC, 1, 0, 0, width, height)
-
-        for a in self.axes:
-            a.draw(pixmap)
-        for t in self._text:
-            t.draw(pixmap)
-            
-        drawable.gdkDrawable.draw_drawable(
-            gc.gdkGC, pixmap.gdkDrawable, 0, 0, 0, 0, width, height)
-        self._pixmap = pixmap
-        self._gc = gc
-
-    def expose_event(self, widget, event):
-        if self._isConfigured and self._isRealized:
-            self._inExpose = True
-
-            if self._doplot:
-                if self._pixmap is not None:
-                    r = event.area
-                    # redraw from pixmap
-                    self.renderer.gdkDrawable.draw_drawable(
-                        self._gc.gdkGC, self._pixmap.gdkDrawable,
-                        r.x, r.y, r.x, r.y, r.width, r.height)
-                else: self.draw()
-            self._inExpose = False
-        return gtk.TRUE
-
-    def focus_in_event(self, widget, event):
-        return gtk.TRUE
-
 
 
         
-    def print_figure(self, filename, dpi=150):
-        root, ext = os.path.splitext(filename)
-        
-        ext = ext.lower()[1:]
-        if not len(ext):
-            filename = filename + '.png'
-            ext = 'png'
-
-        if ext=='png': type = 'png'
-        elif ext in ('jpg', 'jpeg'): type = 'jpeg'
-        elif ext.find('ps')>=0: pass
-        else:
-            error_msg_gtk('Can only save to formats PNG, JPEG, PS or EPS')            
-            return
-
-        if not self._isRealized:
-            self._printQued.append((filename, dpi))
-            return
-
-
-        if ext.find('ps')>=0:
-            oldDPI = self.dpi.get()
-
-            self.dpi.set(72)
-            for a in self.axes: a.resize()
-            fig = switch_backends(self, FigurePS)
-            fig.print_figure(filename, 72)
-            self.dpi.set(oldDPI)
-            for a in self.axes: a.resize()
-            return
-
-        origBounds = self.bbox.get_bounds()
-        origDPI = self.dpi.get()
-        dpi = 72.0/SCREENDPI*dpi
-
-        self.dpi.set(dpi)        
-        width = int(self.figsize[0]*dpi)
-        height = int(self.figsize[1]*dpi)
-        
-        self.bbox.set_bounds(0, 0, width, height)
-
-
-        for a in self.axes:
-            a.resize()
-
-        gc = self.renderer.new_gc()
-        gc.set_foreground('w')
-        gdrawable = self.renderer.gdkDrawable
-        ggc = gc.gdkGC
-        
-        pixmap = RendererGTK(self, gtk.gdk.Pixmap(gdrawable, width, height), self.dpi)
-
-        pixmap.draw_rectangle(gc, True, 0, 0, width, height)
-
-        for ax in self.axes: ax.draw(pixmap)
-        for t in self._text: t.draw(pixmap)
-             
-        pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, 0, 8, width, height)
-        pixbuf.get_from_drawable(pixmap.gdkDrawable, gdrawable.get_colormap(),
-                                 0, 0, 0, 0, width, height)
-
-        try: pixbuf.save(filename, type)
-        except gobject.GError, msg:
-            self.bbox.set_bounds(*origBounds)
-            self.dpi.set(origDPI)
-            self.configure_event(self, 'configure')
-            if self._doplot: self.draw()
-            msg = raise_msg_to_str(msg)
-            # note the error must be displayed here because trapping
-            # the error on a call or print_figure may not work because
-            # printing can be qued and called from realize
-            error_msg_gtk('Could not save figure to %s\n\n%s' % (
-                filename, msg))
-        else:
-            # successful draw, reset
-            self.bbox.set_bounds(*origBounds)
-            self.dpi.set(origDPI)
-            if self._doplot:
-                self.configure_event(self, 'configure')
-                self.draw()
-
-    
-    def _update_renderer(self, widget):
-        colorManager.set_drawing_area(widget)
-        self.renderer = RendererGTK(self, widget.window, self.dpi)
-
-    def realize(self, widget):
-        if widget is None: return 
-        self._update_renderer(widget)
-        self._isRealized = True
-        for fname, dpi in self._printQued:
-            self.print_figure(fname, dpi)
-        self._printQued = []
-        return gtk.TRUE
-
-
-    def is_realized(self):
-        return self._isRealized
-
-
 
 def raise_msg_to_str(msg):
     """msg is a return arg from a raise.  Join with new lines"""
@@ -711,7 +503,7 @@ def draw_if_interactive():
     if matplotlib.is_interactive():
         figManager =  Gcf.get_active()
         if figManager is not None:
-            figManager.figure.draw()
+            figManager.canvas.draw()
             #fig.queue_draw()
 
 
@@ -731,7 +523,7 @@ def show():
 def _quit_after_print_xvfb(*args):
 
     for manager in Gcf.get_all_fig_managers():
-        if len(manager.figure._printQued): break
+        if len(manager.canvas._printQued): break
     else: gtk.mainquit()
 
 def show_xvfb():
@@ -739,51 +531,263 @@ def show_xvfb():
     Print the pending figures only then quit, no screen draw
     """
     for manager in Gcf.get_all_fig_managers():
-        manager.figure.set_do_plot(False)
+        manager.canvas.set_do_plot(False)
         manager.window.show()
         
     gtk.idle_add(_quit_after_print_xvfb)
     gtk.mainloop()                
 
-def new_figure_manager(num, figsize, dpi):
+
+def new_figure_manager(num, *args):
     """
     Create a new figure manager instance
     """
+    figure = Figure(*args)
+
+    canvas = FigureCanvasGTK(figure)
+    return FigureManagerGTK(canvas, num)
+
+class FigureCanvasGTK(gtk.DrawingArea, FigureCanvasBase):
+    def __init__(self, figure):
+        FigureCanvasBase.__init__(self, figure)
+        gtk.DrawingArea.__init__(self)
+
+        self._inExpose = False
+        self._isConfigured = False
+        self._isRealized = False
+        self._gpixmap = None
+        self._doplot = True
+        self._printQued = []
+
+
+        w = figure.bbox.x.interval()
+        h = figure.bbox.y.interval()
+        self.set_size_request(int(w), int(h))
+        self.set_double_buffered(gtk.FALSE)
+        self.connect('key_press_event', self.key_press_event)
+        self.connect('key_release_event', self.key_release_event)
+        self.connect('expose_event', self.expose_event)
+        self.connect('configure_event', self.configure_event)
+        self.connect('realize', self.realize)
+        self.connect('motion_notify_event', self.motion_notify_event)
+        self.connect('button_press_event', self.button_press_event)
+        self.connect('button_release_event', self.button_release_event)
+
+        self.set_events(
+            #gdk.FOCUS_CHANGE_MASK|
+            gdk.KEY_PRESS_MASK|
+            gdk.KEY_RELEASE_MASK|
+            gdk.EXPOSURE_MASK |
+            gdk.LEAVE_NOTIFY_MASK |
+            gdk.BUTTON_PRESS_MASK |
+            gdk.BUTTON_RELEASE_MASK |
+            gdk.POINTER_MOTION_MASK )
+
     
-    thisFig = FigureGTK(figsize, dpi)
-    thisFig.show()
+    def button_press_event(self, widget, event):  pass
+    def button_release_event(self, widget, event): pass
+    def motion_notify_event(self, widget, event): pass
+    def key_press_event(self, widget, event): pass
+    def key_release_event(self, widget, event): pass
+                
+    def configure_event(self, widget, event=None):
 
-    win = gtk.Window()
-    win.set_title("Figure %d" % num)
-    win.set_border_width(5)
-    #def destroy(*args): Gcf.destroy(num)
+        if widget.window is None: return 
+
+        w,h = widget.window.get_size()
+        if w==1 or h==1: return # empty fig
+
+        self.figure.bbox.set_bounds(0, 0, w, h)
+        for a in self.figure.axes: a.resize()
+        
+        if self._isConfigured and self._isRealized:
+            self.draw()
+
+        self._isConfigured = True
+        return gtk.TRUE
+        
+
+    def draw(self):
+        if not self._doplot: return
+        drawable = self.window        
+        if drawable is None: return
+        
+        width = int(self.figure.bbox.x.interval())
+        height = int(self.figure.bbox.y.interval())
+
+        gpixmap = gtk.gdk.Pixmap( drawable, width, height)
+        pixmap = RendererGTK(self, gpixmap, self.figure.dpi)
+
+        self.figure.draw(pixmap)
+
+        ggc = drawable.new_gc()
+        drawable.draw_drawable(
+            ggc, gpixmap, 0, 0, 0, 0, width, height)
+        self._gpixmap = gpixmap
+        self._ggc = ggc
+
+    def expose_event(self, widget, event):
+        if widget.window is None: return 
+
+        if self._gpixmap is not None:
+            r = event.area
+            # redraw from pixmap
+            widget.window.draw_drawable(
+                self._ggc, self._gpixmap,
+                r.x, r.y, r.x, r.y, r.width, r.height)
+        else: self.draw()
+
+        return gtk.TRUE
 
 
-    vbox = gtk.VBox(spacing=3)
-    win.add(vbox)
-    vbox.show()
-    vbox.pack_start(thisFig)
+    def print_figure(self, filename, dpi=150,
+                     facecolor='w', edgecolor='w',
+                     orientation='portrait'):
 
-    toolbar = NavigationToolbar( thisFig, win)
-    toolbar.show()
-    vbox.pack_start(toolbar, gtk.FALSE, gtk.FALSE )
+        if is_string_like(filename): isFileName = True
+        else: isFileName = False
 
-    figManager = FigureManagerGTK(thisFig, num, win, vbox, toolbar)
-    def destroy(*args): Gcf.destroy(num)
-    win.connect("destroy", destroy)
+        if isFileName:
+            root, ext = os.path.splitext(filename)        
+            ext = ext.lower()[1:]
+            if not len(ext):
+                filename = filename + '.png'
+                ext = 'png'
 
-    if matplotlib.is_interactive():
-        win.show()
+            if ext=='png': type = 'png'
+            elif ext in ('jpg', 'jpeg'): type = 'jpeg'
+            elif ext.find('ps')>=0: pass
+            else:
+                error_msg_gtk('Can only save to formats PNG, JPEG, PS or EPS')            
+                return
+        else: type='png'
+        
+        if not self._isRealized:
+            self._printQued.append((filename, dpi, facecolor, edgecolor))
+            return
+
+
+        if isFileName and ext.find('ps')>=0:
+            
+            # enable ps save from gtk backend do the import here so we
+            # don't have to import afm unless necessary
+            from backend_ps import FigureCanvasPS
+
+            origDPI = self.figure.dpi.get()
+            ps = self.switch_backends(FigureCanvasPS)
+            ps.figure.dpi.set(72)
+            for a in ps.figure.axes: a.resize()
+            ps.print_figure(filename, 72, facecolor, edgecolor)
+            self.figure.dpi.set(origDPI)
+            for a in self.figure.axes: a.resize()
+            return
+
+        origfacecolor = self.figure.get_facecolor()
+        origedgecolor = self.figure.get_edgecolor()
+        self.figure.set_facecolor(facecolor)
+        self.figure.set_edgecolor(edgecolor)
+
+        origBounds = self.figure.bbox.get_bounds()
+        origDPI = self.figure.dpi.get()
+        dpi = 72.0/PIXELS_PER_INCH*dpi
+
+        self.figure.dpi.set(dpi)        
+        width = int(self.figure.figsize[0]*dpi)
+        height = int(self.figure.figsize[1]*dpi)
+        
+        self.figure.bbox.set_bounds(0, 0, width, height)
+
+
+        for a in self.figure.axes: a.resize()
+
+        gdrawable = self.window
+        ggc = gdrawable.new_gc()
+
+        gpixmap = gtk.gdk.Pixmap(gdrawable, width, height)
+        pixmap = RendererGTK(
+            self, gpixmap, self.figure.dpi)
+
+        self.figure.draw(pixmap)
+        
+        pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, 0, 8, width, height)
+        pixbuf.get_from_drawable(gpixmap, gdrawable.get_colormap(),
+                                 0, 0, 0, 0, width, height)
+
+        self.figure.set_facecolor(origfacecolor)
+        self.figure.set_edgecolor(origedgecolor)
+        self.figure.dpi.set(origDPI)
+        self.figure.bbox.set_bounds(*origBounds)
+        self.configure_event(self, 'configure')
+
+        try: pixbuf.save(filename, type)
+        except gobject.GError, msg:
+            msg = raise_msg_to_str(msg)
+            # note the error must be displayed here because trapping
+            # the error on a call or print_figure may not work because
+            # printing can be qued and called from realize
+            if isFileName:
+                error_msg_gtk('Could not save figure to %s\n\n%s' % (
+                    filename, msg))
+            else:
+                error_msg_gtk('Could not save figure\n%s' % msg)
+                
+    def realize(self, widget):
+        if widget is None: return 
+        colorManager.set_drawing_area(widget)
+
+        self._isRealized = True
+        for fname, dpi, facecolor, edgecolor in self._printQued:
+            self.print_figure(fname, dpi, facecolor, edgecolor)
+        self._printQued = []
+        return gtk.TRUE
+
+
+    def is_realized(self):
+        return self._isRealized
+
+
+    def set_do_plot(self, b):
+        'True if you want to render to screen, False is hardcopy only'
+        self._doplot = b
 
-    return figManager
 
 class FigureManagerGTK(FigureManagerBase):
-    def __init__(self, figure, num, window, vbox, toolbar):
-        FigureManagerBase.__init__(self, figure, num)
+    """
+    Public attributes
+
+    canvas      : The FigureCanvas instance
+    num         : The Figure number
+    toolbar     : The gtk.Toolbar
+    window      : The gtk.Window
+    
+    """
+    def __init__(self, canvas, num):
+        FigureManagerBase.__init__(self, canvas, num)
+        
+
+        self.window = gtk.Window()
+        self.window.set_title("Figure %d" % num)
+        self.window.set_border_width(5)
+
+        vbox = gtk.VBox(spacing=3)
+        self.window.add(vbox)
+        vbox.show()
+
+        self.canvas.show()
+        vbox.pack_start(self.canvas, gtk.TRUE, gtk.TRUE)
+
+        # must be inited after the window, drawingArea and figure
+        # attrs are set
+        self.toolbar = NavigationToolbar( canvas, self.window )
+        self.toolbar.show()
+        vbox.pack_end(self.toolbar, gtk.FALSE, gtk.FALSE )
+
+        def destroy(*args): Gcf.destroy(num)
+        self.window.connect("destroy", destroy)
+
+        if matplotlib.is_interactive():
+            self.window.show()
 
-        self.window = window
-        self.vbox = vbox
-        self.toolbar = toolbar
 
     def add_subplot(self, *args, **kwargs):
         a = FigureManagerBase.add_subplot(self, *args, **kwargs)
@@ -795,11 +799,6 @@ class FigureManagerGTK(FigureManagerBase):
         self.toolbar.update()
         return a
     
-    def set_current_axes(self, a):
-        if a not in self.axes.values():
-            error_msg_gtk('Axes is not in current figure')
-        FigureManagerBase.set_current_axes(self, a)
-
     def destroy(self, *args):
         self.window.destroy()
         if Gcf.get_num_fig_managers()==0 and not matplotlib.is_interactive():
@@ -807,6 +806,8 @@ class FigureManagerGTK(FigureManagerBase):
 
 
         
+
+        
 class Dialog_MeasureTool(gtk.Dialog):
     def __init__(self):
         gtk.Dialog.__init__(self)
@@ -841,17 +842,24 @@ class Dialog_MeasureTool(gtk.Dialog):
 
 
 class NavigationToolbar(gtk.Toolbar):
-    
-    def __init__(self, figure, win=None):
-        """
-        figure is the Figure instance that the toolboar controls
+    """
+    Public attriubutes
 
-        win, if not None, is the gtk.Window the Figure is embedded in
+      canvas   - the FigureCanvas  (gtk.DrawingArea)
+      win   - the gtk.Window
+
+    """
+    def __init__(self, canvas, window=None):
+        """
+        figManager is the FigureManagerGTK instance that contains the
+        toolbar, with attributes figure, window and drawingArea
         
         """
         gtk.Toolbar.__init__(self)
-        self.win = win
-        self.figure = figure
+        
+        self.canvas = canvas
+        self.win = window
+        
         iconSize = gtk.ICON_SIZE_SMALL_TOOLBAR
         self.set_border_width(5)
         self.set_style(gtk.TOOLBAR_ICONS)
@@ -964,8 +972,10 @@ class NavigationToolbar(gtk.Toolbar):
 
 
         def destroy_clicked(button):
+            win = self.win
             if win is not None: win.destroy()
             else: gtk.mainquit()
+
         iconw = gtk.Image()
         iconw.set_from_stock(gtk.STOCK_QUIT, iconSize)
         self.bQuit = self.append_item(
@@ -1035,7 +1045,8 @@ class NavigationToolbar(gtk.Toolbar):
 
         for a in self._active:
             a.panx(direction)
-        self.figure.draw()
+        self.canvas.draw()
+
         return gtk.TRUE
     
     def pany(self, button, arg):
@@ -1046,7 +1057,7 @@ class NavigationToolbar(gtk.Toolbar):
             else: direction=-1
         for a in self._active:
             a.pany(direction)
-        self.figure.draw()
+        self.canvas.draw()
         return gtk.TRUE
     
     def zoomx(self, button, arg):
@@ -1058,7 +1069,7 @@ class NavigationToolbar(gtk.Toolbar):
 
         for a in self._active:
             a.zoomx(direction)
-        self.figure.draw()
+        self.canvas.draw()
         return gtk.TRUE
 
     def zoomy(self, button, arg):
@@ -1070,7 +1081,7 @@ class NavigationToolbar(gtk.Toolbar):
 
         for a in self._active:
             a.zoomy(direction)
-        self.figure.draw()
+        self.canvas.draw()
         return gtk.TRUE
 
     def menu_clicked(self, button):
@@ -1084,15 +1095,17 @@ class NavigationToolbar(gtk.Toolbar):
             fname = fs.get_filename()
             self.lastDir = os.path.dirname(fname)
             fs.destroy()
-            try: self.figure.print_figure(fname)
+            try: self.canvas.print_figure(fname)
             except IOError, msg:                
                 err = '\n'.join(map(str, msg))
                 msg = 'Failed to save %s: Error msg was\n\n%s' % (
                     fname, err)
+                error_msg_gtk(msg)
             
         fs = gtk.FileSelection(title='Save the figure')
-        if self.win is not None:
-            fs.set_transient_for(self.win)
+        win = self.win
+        if win is not None:
+            fs.set_transient_for(win)
         try: self.lastDir
         except AttributeError: pass
         else: fs.set_filename(self.lastDir + os.sep)
@@ -1103,7 +1116,7 @@ class NavigationToolbar(gtk.Toolbar):
 
 
     def update(self):
-        self._axes = self.figure.get_axes()
+        self._axes = self.canvas.figure.axes
         self.set_active(range(len(self._axes)))
         if len(self._axes)>1:
 
@@ -1122,6 +1135,5 @@ class NavigationToolbar(gtk.Toolbar):
             self.omenu.set_menu(menu)
         self.set_active(range(len(self._axes)))
 
-FigureManager = FigureManagerBase
-Figure = FigureGTK
+FigureManager = FigureManagerGTK
 error_msg = error_msg_gtk
