@@ -3,21 +3,22 @@ This module contains all the 2D line class which can draw with a
 variety of line styles, markers and colors
 """
 
-from __future__ import generators
+# TODO: expose cap and join style attrs
 from __future__ import division
 
 import sys
 
 from Numeric import Float, alltrue, arange, array, logical_and,\
-     nonzero, reshape, resize, searchsorted, take, Int, asarray, ones
+     nonzero, searchsorted, take, asarray, ones
 
 from artist import Artist
 from cbook import True, False
-from transforms import Bound1D, Bound2D, Transform
+from colors import colorConverter
+from transforms import Bound2D, Transform
 
 lineStyles = {'-':1, '--':1, '-.':1, ':':1}
 lineMarkers =    {'.':1, ',':1, 'o':1, '^':1,
-                  'v':1, '<':1, '>':1, 's':1, '+':1}
+                  'v':1, '<':1, '>':1, 's':1, '+':1, 'x':1}
 
 class Line2D(Artist):
 
@@ -26,6 +27,7 @@ class Line2D(Artist):
                  color='b', 
                  marker=None, markersize=6,
                  markeredgecolor=None, markerfacecolor=None,
+                 antialiased=True,
                  transx=Transform(),
                  transy=Transform(),
                  ):
@@ -35,7 +37,7 @@ class Line2D(Artist):
         xdata is a sequence of x data
         ydata is a sequence of y data
         linewidth is the width of the line in points
-        linestle is one of lineStyls
+        linestyle is one of lineStyles
         marker is one of lineMarkers or None
         markersize is the size of the marker in points
         markeredgecolor  and markerfacecolor can be any color arg 
@@ -51,6 +53,7 @@ class Line2D(Artist):
         self._linestyle = linestyle
         self._linewidth = linewidth
         self._color = color
+        self._antialiased = antialiased
         self._markersize = markersize
         if markeredgecolor is None: markeredgecolor=color
         
@@ -78,6 +81,7 @@ class Line2D(Artist):
             '>'  : self._draw_triangle_right, 
             's'  : self._draw_square,
             '+'  : self._draw_plus,
+            'x'  : self._draw_x,
             None : self._draw_nothing
             }
 
@@ -164,8 +168,10 @@ class Line2D(Artist):
 
         gc = renderer.new_gc()
         gc.set_foreground(self._color)
+        gc.set_antialiased(self._antialiased)
         gc.set_linewidth(self._linewidth)
-        if self._clipOn:
+        gc.set_alpha(self._alpha)
+        if self.get_clip_on():
             gc.set_clip_rectangle(self.bbox.get_bounds())
         self._lineFunc(renderer, gc, xt, yt)
 
@@ -177,20 +183,16 @@ class Line2D(Artist):
                 gc.set_clip_rectangle(self.bbox.get_bounds())
             self._markerFunc(renderer, gc, xt, yt)
 
-    def _derived_draw(self, renderer, gc, x, y):
-        raise NotImplementedError, 'Line2D is a pure base class.  ' + \
-              'You must instantiate a derived class'
-
-
+    def get_antialiased(self): return self._antialiased
+    def get_color(self): return self._color
+    def get_linestyle(self): return self._linestyle
+    def get_linewidth(self): return self._linewidth
+    def get_marker(self): return self._marker
+    def get_markeredgecolor(self): return self._markeredgecolor
+    def get_markerfacecolor(self): return self._markerfacecolor
+    def get_markersize(self): return self._markersize
     def get_xdata(self): return self._x
     def get_ydata(self):  return self._y
-    def get_linewidth(self): return self._linewidth
-    def get_linestyle(self): return self._linestyle
-    def get_marker(self): return self._marker
-    def get_markersize(self): return self._markersize
-    def get_markerfacecolor(self): return self._markerfacecolor
-    def get_markeredgecolor(self): return self._markeredgecolor
-    def get_color(self): return self._color
 
         
     def _set_clip(self):
@@ -219,7 +221,7 @@ class Line2D(Artist):
             else:
                 indx = nonzero(
                     logical_and( self._x>=self._xmin,
-                                       self._x<=self._xmax ))
+                                 self._x<=self._xmax ))
         
 
         self._xc = take(self._x, indx)
@@ -241,6 +243,12 @@ class Line2D(Artist):
         self._xc = take(self._xc, indy)
         self._yc = take(self._yc, indy)
 
+    def set_antialiased(self, b):
+        """
+        True if line should be drawin with antialiased rendering
+        """            
+        self._antialiased = b
+
     def set_color(self, color):
         self._color = color
 
@@ -283,28 +291,23 @@ class Line2D(Artist):
     
     def _draw_solid(self, renderer, gc, xt, yt):
         gc.set_linestyle('solid')
+        gc.set_capstyle('projecting')   
         renderer.draw_lines(gc, xt,yt)
 
     def _draw_dashed(self, renderer, gc, xt, yt):
-        dashes = self.dpi.get()/72.0*array([6.0, 6.0])
-        #gc.set_dashes(0, dashes)  #  gdk.LINE_ON_OFF_DASH
         gc.set_linestyle('dashed')
-        gc.set_capstyle('butt')   #  gdk.CAP_BUTT
-        gc.set_joinstyle('miter')  #  gdk.JOIN_MITER
+        gc.set_capstyle('butt')   
+        gc.set_joinstyle('miter') 
         renderer.draw_lines(gc, xt, yt)
 
     def _draw_dash_dot(self, renderer, gc, xt, yt):
-        dashes = self.dpi.get()/72.0*array([3.0, 5.0, 1.0, 5.0])
-        #gc.set_dashes(0, dashes)  #3 on, 5 off, 1 on, 5 off
         gc.set_linestyle('dashdot')
         gc.set_capstyle('butt')   
         gc.set_joinstyle('miter') 
         renderer.draw_lines(gc, xt, yt)
 
     def _draw_dotted(self, renderer, gc, xt, yt):
-        dashes = self.dpi.get()/72.0*array([1.0, 2.0])
         gc.set_linestyle('dotted')
-        #gc.set_dashes(0, dashes)  
         gc.set_capstyle('butt')   
         gc.set_joinstyle('miter') 
         renderer.draw_lines(gc, xt, yt)
@@ -313,71 +316,86 @@ class Line2D(Artist):
         for (x,y) in zip(xt, yt):
             renderer.draw_arc(gc, None, x, y, 1, 1, 0.0, 360.0)
 
-
     def _draw_pixel(self, renderer, gc, xt, yt):
         for (x,y) in zip(xt, yt):
             renderer.draw_point(gc, x, y)
 
 
     def _draw_circle(self, renderer, gc, xt, yt):
-        w = h = self._markersize*self.dpi.get()/72.0
+        w = h = renderer.points_to_pixels(self._markersize)
+
+
+        rgbFace = self._get_rgb_face()
         for (x,y) in zip(xt, yt):
-            renderer.draw_arc(gc, self._markerfacecolor,
+            renderer.draw_arc(gc, rgbFace,
                               x, y, w, h, 0.0, 360.0)
 
     def _draw_triangle_up(self, renderer, gc, xt, yt):
-        offset = 0.5*self._markersize*self.dpi.get()/72.0
         
+        offset = 0.5*renderer.points_to_pixels(self._markersize)
+        rgbFace = self._get_rgb_face()
         for (x,y) in zip(xt, yt):
             verts = ( (x, y+offset),
                       (x-offset, y-offset),
                       (x+offset, y-offset) )
-            renderer.draw_polygon(gc, self._markerfacecolor, verts)
+            renderer.draw_polygon(gc, rgbFace, verts)
 
 
     def _draw_triangle_down(self, renderer, gc, xt, yt):
-        offset = 0.5*self._markersize*self.dpi.get()/72.0
+        offset = 0.5*renderer.points_to_pixels(self._markersize)
+        rgbFace = self._get_rgb_face()
         for (x,y) in zip(xt, yt):            
             verts = ( (x-offset, y+offset),
                       (x+offset, y+offset),
                       (x, y-offset))
-            renderer.draw_polygon(gc, self._markerfacecolor, verts)
+            renderer.draw_polygon(gc, rgbFace, verts)
 
     def _draw_triangle_left(self, renderer, gc, xt, yt):
-        offset = 0.5*self._markersize*self.dpi.get()/72.0
+        offset = 0.5*renderer.points_to_pixels(self._markersize)
+        rgbFace = self._get_rgb_face()
         for (x,y) in zip(xt, yt):            
             verts = ( (x-offset, y),
                       (x+offset, y-offset),
                       (x+offset, y+offset))
-            renderer.draw_polygon(gc, self._markerfacecolor, verts)
+            renderer.draw_polygon(gc, rgbFace, verts)
 
 
     def _draw_triangle_right(self, renderer, gc, xt, yt):
-        offset = 0.5*self._markersize*self.dpi.get()/72.0        
+        offset = 0.5*renderer.points_to_pixels(self._markersize)
+        rgbFace = self._get_rgb_face()
         for (x,y) in zip(xt, yt):            
             verts = ( (x+offset, y),
                       (x-offset, y-offset),
                       (x-offset, y+offset))
-            renderer.draw_polygon(gc, self._markerfacecolor, verts)
+            renderer.draw_polygon(gc, rgbFace, verts)
 
             
 
     def _draw_square(self, renderer, gc, xt, yt):
-        side = self.dpi.get()/72.0*self._markersize
+        side = renderer.points_to_pixels(self._markersize)
         offset = side*0.5
+        rgbFace = self._get_rgb_face()
         for (x,y) in zip(xt, yt):            
             renderer.draw_rectangle(
-                gc, self._markerfacecolor,
+                gc, rgbFace,
                 x-offset, y-offset, side, side) 
 
     def _draw_plus(self, renderer, gc, xt, yt):
-        offset = self._markersize*self.dpi.get()/72.0
+        offset = 0.5*renderer.points_to_pixels(self._markersize)
         # todo: unify marker size args; use default vals for line
         # sytles in dict?
         for (x,y) in zip(xt, yt):
             renderer.draw_line(gc, x-offset, y, x+offset, y)
             renderer.draw_line(gc, x, y-offset, x, y+offset)
 
+    def _draw_x(self, renderer, gc, xt, yt):
+        offset = 0.5*renderer.points_to_pixels(self._markersize)
+        # todo: unify marker size args; use default vals for line
+        # sytles in dict?
+        for (x,y) in zip(xt, yt):
+            renderer.draw_line(gc, x-offset, y-offset, x+offset, y+offset)
+            renderer.draw_line(gc, x-offset, y+offset, x+offset, y-offset)
+
     def copy_properties(self, line):
 
         self._linestyle = line._linestyle
@@ -392,3 +410,8 @@ class Line2D(Artist):
         self._lineFunc = line._lineStyles[line._linestyle]
         self._markerFunc = line._markers[line._marker]
         self._useDataClipping = line._useDataClipping
+
+    def _get_rgb_face(self):
+        if self._markerfacecolor is None: rgbFace = None
+        else: rgbFace = colorConverter.to_rgb(self._markerfacecolor)
+        return rgbFace
