@@ -8,6 +8,8 @@ Plotting commands
   axes     - Create a new axes
   axis     - Set or return the current axis limits
   bar      - make a bar chart
+  cla      - clear current axes
+  clf      - clear a figure window
   close    - close a figure window
   cohere   - make a plot of coherence
   csd      - make a plot of cross spectral density
@@ -120,13 +122,11 @@ from FFT import fft
 import _matlab_helpers
 import mlab  #so I can override hist, psd, etc...
 
-from axes import Subplot, Axes
-from backends import Figure, new_figure_manager, error_msg, \
+from axes import Axes
+from backends import new_figure_manager, error_msg, \
      draw_if_interactive, show
-from cbook import flatten, is_string_like
-from colormap import ColormapJet
-from lines import Line2D
-from patches import Rectangle
+from cbook import flatten, is_string_like, exception_to_str
+from figure import Figure
 
 def colors():
     """
@@ -172,7 +172,9 @@ def plotting():
     axes     - Create a new axes
     axis     - Set or return the current axis limits
     bar      - make a bar chart
-    close    - close a figure window
+    cla      - clear the current axes    
+    clf      - clear the current figure window
+    close    - close the current figure window
     cohere   - make a plot of coherence
     csd      - make a plot of cross spectral density
     errorbar - make an errorbar graph
@@ -212,7 +214,7 @@ def get_current_fig_manager():
 def get_plot_commands(): return ['axis', 'axes', 'subplot', 'plot',
     'set', 'get', 'title', 'xlabel', 'ylabel', 'text', 'scatter',
     'errorbar', 'bar', 'hist', 'figure', 'gca', 'gcf', 'close',
-    'pcolor', 'loglog', 'semilogx', 'semilogy', 'psd', 'csd', 'cohere']
+    'pcolor', 'loglog', 'semilogx', 'semilogy', 'psd', 'csd', 'cohere', 'clf', 'cla']
 
 
 def raise_msg_to_str(msg):
@@ -229,6 +231,8 @@ def axis(*v):
 
     axis(v) where v= [xmin xmax ymin ymax] sets the min and max of the
     x and y axis limits
+
+    axis('off') turns off the axis lines and labels
     """
     
     if len(v)==1 and is_string_like(v[0]):
@@ -297,31 +301,37 @@ def axes(*args, **kwargs):
 
 def bar(*args, **kwargs):
     """
-    bar(self, x, y, width=0.8)
+    BAR(left, height)
 
-      Make a bar plot with rectangles at x, x+width, 0, y x and y are
-      Numeric arrays
+    Make a bar plot with rectangles at
+      left, left+width, 0, height
+    left and height are Numeric arrays
 
-    bar(x, y, width=0.8, color='b', yerr=None, xerr=None, capsize=3)
-
-      xerr and yerr, if not None, will be used to generate errorbars
-      on the bar chart for the x and y axes, respectively.  See
-      errorbar for more info on the xerr and yerr optional arguments.
-
-      color can be a matplotlib color string or tuple, or it can be
-      a len(x) list of matplotlib color strings or tuples
-
-      capsize gives the length of the errorbar; see errorbar for more
-      info.
-      
     Return value is a list of Rectangle patch instances
-    """
 
+    BAR(left, height, width, bottom,
+        color, yerr, xerr, capsize, yoff)
+
+    xerr and yerr, if not None, will be used to generate errorbars
+    on the bar chart
+
+    color specifies the color of the bar
+
+    capsize determines the length in points of the error bar caps
+
+
+    The optional arguments color, width and bottom can be either
+    scalars or len(x) sequences
+
+    This enables you to use bar as the basis for stacked bar
+    charts, or candlestick plots
+    """
+    
     try: patches =  gca().bar(*args, **kwargs)
-    except ValueError, msg:
-        msg = raise_msg_to_str(msg)
-        error_msg(msg)
-        raise RuntimeError, msg
+    except Exception, msg:
+        s = exception_to_str(msg)
+        error_msg(s)
+        raise RuntimeError('Could not exectute bar')
     draw_if_interactive()
     return patches
 
@@ -360,6 +370,23 @@ def close(*args):
         msg = 'close takes 0 or 1 arguments'
         popup_err(msg)
 
+
+def cla():
+    """
+    Clear the current axes
+    """
+    gca().cla()
+    draw_if_interactive()
+    
+def clf():
+    """
+    Clear the current figure
+    """
+    manager = get_current_fig_manager()
+    manager.clf()
+    draw_if_interactive()
+
+
 def cohere(x, y, NFFT=256, Fs=2, detrend=mlab.detrend_none,
            window=mlab.window_hanning, noverlap=0):
     """
@@ -417,7 +444,6 @@ def csd(x, y, NFFT=256, Fs=2, detrend=mlab.detrend_none,
         Procedures, John Wiley & Sons (1986)
 
     """
-
     try: ret =  gca().csd(x, y, NFFT, Fs, detrend, window, noverlap)
     except ValueError, msg:
         msg = raise_msg_to_str(msg)
@@ -452,7 +478,7 @@ def errorbar(x, y, yerr=None, xerr=None, fmt='b-', capsize=3):
         draw_if_interactive()
         return ret
 
-def figure(num=1, figsize=(8,6), dpi=72):
+def figure(num=1, figsize=(8,6), dpi=72, facecolor=0.75, edgecolor='w'):
     """
     Create a new figure and return a handle to it
 
@@ -468,10 +494,10 @@ def figure(num=1, figsize=(8,6), dpi=72):
 
     figManager = _matlab_helpers.Gcf.get_fig_manager(num)
     if figManager is None:
-        figManager = new_figure_manager(num, figsize, dpi)
+        figManager = new_figure_manager(num, figsize, dpi, facecolor, edgecolor)
         _matlab_helpers.Gcf.set_active(figManager)
     
-    return figManager.figure
+    return figManager.canvas.figure
 
 
 def gca():
@@ -498,7 +524,7 @@ def gcf():
     
     figManager = _matlab_helpers.Gcf.get_active()
     if figManager is not None:
-        return figManager.figure
+        return figManager.canvas.figure
     else:
         return figure()
 
@@ -778,18 +804,23 @@ def psd(x, NFFT=256, Fs=2, detrend=mlab.detrend_none,
 
 
 
-def savefig(fname, dpi=150):
+def savefig(*args, **kwargs):
     """
+    def savefig(fname, dpi=150, facecolor='w', edgecolor='w',
+                orientation='portrait'):
     Save the current figure to filename fname.  dpi is the resolution
     in dots per inch.
 
     Output file types currently supported are jpeg and png and will be
     deduced by the extension to fname
-    
-    """
-    # print_figure does it's own error handling because of queing
-    #todo: allow default backend dependent extensions
-    gcf().print_figure(fname, dpi)
+
+    facecolor and edgecolor are the colors os the figure rectangle
+
+    orientation is either 'landscape' or 'portrait' - not supported on
+    all backends; currently only on postscript output."""
+
+    manager = get_current_fig_manager()
+    manager.canvas.print_figure(*args, **kwargs)
 
 def scatter(*args, **kwargs):
     """
@@ -875,17 +906,20 @@ def set(h, *args, **kwargs):
         funcvals.append((args[i], args[i+1]))
     funcvals.extend(kwargs.items())
 
+    ret = []
     for o in h:
         for s, val in funcvals:
             s = s.lower()
             funcName = "set_%s"%s
             func = getattr(o,funcName)        
-            try: func(val)
+            try: ret.extend( [func(val)] )
             except ValueError, msg:
                 msg = raise_msg_to_str(msg)
                 error_msg(msg)
                 raise RuntimeError, msg
+        
     draw_if_interactive()
+    return [x for x in flatten(ret)]
 
 def subplot(*args, **kwargs):
     """
@@ -1064,3 +1098,23 @@ def ylabel(s, *args, **kwargs):
     draw_if_interactive()
     return l
 
+
+def table(*args, **kwargs):
+    """
+    table(cellText=None, cellColours=None,
+          cellLoc='right', colWidths=None,
+          rowLabels=None, rowColours=None, rowLoc='left',
+          colLabels=None, colColours=None, colLoc='center',
+          loc='bottom', bbox=None):
+
+    Add a table to the current axes.  Returns a table instance.  For
+    finer grained control over tables, use the Table class and add it
+    to the axes with add_table.
+
+    Thanks to John Gill for providing the class and table.
+
+    """
+    t = gca().table(*args, **kwargs)
+    draw_if_interactive()
+    return t
+    
